/****************************************************************************
 * Universidade Federal Fluminense - Instituto de Computao                *
 * Author: Bruno Siqueira                                                   *
 * brunos@midiacom.uff.br                                                   *
 * 2013                                                                     *
 ****************************************************************************/

///
/// \file	LIBR_pkt.h
/// \brief	This file contains all declarations of LIBR message,
///		including all related contants and macros.
///

#ifndef __LIBR_pkt_h__
#define __LIBR_pkt_h__

#include <packet.h>
#include <iostream>

/// Accessor to the LIBR packet.
#define HDR_LIBR_PKT(p) hdr_LIBR_pkt::access(p)

/// Fixed lengh of packet header in bytes
#define LIBR_HDR_LEN		12

/// Variable lengh of neighbor message in bytes
#define LIBR_NB_LEN			3

/// Maximum number of neighbors
#define LIBR_MAX_NB			32

/// Packet window for ETX computation
#define LIBR_PKT_WINDOW		(u_int8_t) 25


/// Type of LIBR ID
typedef unsigned char libr_id_t;

/// Auxiliary struct (related to gateways) which is part of the LIBR message.
struct LIBR_msg_gw {

	libr_id_t	id_;  		///<  8 bits: gateway ID
	u_int8_t	seq_;  		///<  8 bits: sequence number of route
	u_int8_t    next_hop_; 	///<  8 bits: ID of next hop
	float		metric_;	///< 16 bits: distance-vector metric to gateway
		
	inline libr_id_t&	id()		{ return id_; }
	inline u_int8_t&	seq()		{ return seq_; }
	inline libr_id_t&	next_hop()	{ return next_hop_; }
	inline float&		metric()	{ return metric_; }

};

/// Auxiliary struct (related to neighbors) which is part of the LIBR message.
struct LIBR_msg_nb {

 	libr_id_t	id_;  		///<  8 bits: neighbor ID
 	float		delivery_;	///< 16 bits: delivery ratio at neighbor
		
	inline libr_id_t&	id()		{ return id_; }
	inline float&		delivery()	{ return delivery_; }

};

/// LIBR message
struct hdr_LIBR_pkt {
	
	libr_id_t		pkt_src_;				///<  8 bits: node-ID which originated this packet
	u_int8_t		pkt_seq_num_;			///<  8 bits: packet sequence number
	LIBR_msg_gw		msg_pri_gw_;			///< 40 bits: primary gateway
	LIBR_msg_gw		msg_sec_gw_;			///< 40 bits: secondary gateway
	LIBR_msg_nb		msg_nb_[LIBR_MAX_NB];	///< Variable size: array of neighbors

	inline libr_id_t&		pkt_src()		{ return pkt_src_; }
	inline u_int8_t&		pkt_seq_num()	{ return pkt_seq_num_; }
	inline LIBR_msg_gw&		msg_pri_gw()	{ return msg_pri_gw_; }
	inline LIBR_msg_gw&		msg_sec_gw()	{ return msg_sec_gw_; }
	inline LIBR_msg_nb&		msg_nb(int i)	{ return msg_nb_[i]; }

	static int			offset_;
	inline static int&	offset()		{ return offset_; }
	inline static hdr_LIBR_pkt*	access(const Packet* p) {
		return (hdr_LIBR_pkt*)p->access(offset_);
	}

};

#endif  /* __LIBR_pkt_h__ */
