/****************************************************************************
 * Universidade Federal Fluminense - Instituto de Computao                *
 * Author: Bruno Siqueira                                                   *
 * brunos@midiacom.uff.br                                                   *
 * 2013                                                                     *
 ****************************************************************************/

///
/// \file	LIBR_rtable.h
/// \brief	Header file for routing table's related stuff.
///

#ifndef __LIBR_rtable_h__
#define __LIBR_rtable_h__


#include <ip.h>
#include <trace.h>
#include <map>
#include <iostream>

///
/// \brief Defines rtable_t as a map of nsaddr_t (red-black tree), whose key
/// is the destination address.
///
/// The routing table is thus defined as pairs: [dest address, next hop]. Each element
/// of the pair can be accesed via "first" and "second" members.
///
typedef std::map<nsaddr_t, nsaddr_t> rtable_t;

///
/// \brief This class is a representation of the LIBR's Routing Table.
///
class LIBR_rtable {

	rtable_t rt_;	///< Data structure for the routing table.

public:

	LIBR_rtable();
	void		print(Trace*);
	void		clear();
	void		rm_entry(nsaddr_t);
	void		add_entry(nsaddr_t, nsaddr_t);
	nsaddr_t	lookup(nsaddr_t);
	u_int32_t	size();

};

#endif  /* __LIBR_rtable_h__ */
